require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes rex_client]].flatten.join('/').to_s

RSpec.describe Hermes::RexClient, type: :helper do

  it 'get user token' do
    VCR.use_cassette("rex_get_token", :match_requests_on => [:method, :uri, :path]) do
      expect(Hermes::RexClient::Client.new.get_token).not_to be_empty
    end
  end

  it 'find entity by id ' do
    VCR.use_cassette("rex_find_entity_by_id", :match_requests_on => [:method, :uri, :path]) do
      expect(Hermes::RexClient::Client.new.find_entity_by_id('795e0ad7-18ca-41b1-b804-4c4e9e00d673')).not_to be_empty
    end
  end

  it 'find lessor by id ' do
    VCR.use_cassette("rex_find_lessor_by_id", :match_requests_on => [:method, :uri, :path]) do
      expect(Hermes::RexClient::Client.new.find_lessor_by_id('a397c0d2-5695-43ef-abff-7d2eb9d2b62b')).not_to be_empty
    end
  end

  it 'find structure by id' do
    VCR.use_cassette("rex_find_structure_by_id", :match_requests_on => [:method, :uri, :path]) do
      expect(Hermes::RexClient::Client.new.find_structure_by_id('a397c0d2-5695-43ef-abff-7d2eb9d2b62b')).not_to be_empty
    end
  end

  it 'search entity on siret name ' do
    VCR.use_cassette("rex_search_entities", :match_requests_on => [:method, :uri, :path]) do
      expect(Hermes::RexClient::Client.new.search_entities({'fields.siret' => '39893387900015'}).length).to eq 1
    end
  end

  it 'search lessor on siret ' do
    VCR.use_cassette("rex_search_lessors", :match_requests_on => [:method, :uri, :path]) do
      expect(Hermes::RexClient::Client.new.search_lessors({'fields.siret' => '52844116500016'}).length).to eq 1
    end
  end

  it 'search structure on id_cristal ' do
    VCR.use_cassette("rex_search_structures", :match_requests_on => [:method, :uri, :path]) do
      expect(Hermes::RexClient::Client.new.search_structures({'fields.id_cristal' => '|52313'}).length).to eq 1
    end
  end

  describe 'get_page' do 
    let!(:client) { Hermes::RexClient::Client.new }

    before do 
      client.base_url = "https://idrex-int.be-idys.com/id-rex-front-ws/context/1bf39a51-d57b-43fe-ad50-d426b9533b22"
      client.ed_entity = "804d1ddb-b508-40fa-8b00-27be5aba1bce"
      client.ed_lessor = "64467464-8890-4429-94fa-e0ec34134dc5"
      client.ed_contact = "864e35c1-862a-4e8e-8f13-deab5ccdbe73"
      client.client_id = "id-rex_back"
      client.client_secret = "6f593606-2306-4617-82b4-d0061a0f2d04"
      client.password = "beysALSREX2024"
      client.username = "idrex_test_user_user_rw"
      client.logger = Logger.new('log/rex-test.log')
    end 

    it 'when receiving 200' do 
      VCR.use_cassette("rex_get_page_200", :match_requests_on => [:method, :uri, :path]) do
        page = client.get_page({}, 'entity', 1)
        expect(page[:items].length).to eq 10
      end 
    end 

    it 'when receiving an error' do 
      VCR.use_cassette("rex_get_page_error", :match_requests_on => [:method, :uri, :path]) do
        page = client.get_page({}, 'entity', 9999999)
        expect(page[:error]).to include("RestClient::BadRequest: 400")
      end 
    end 

    it 'when receiving a 401' do 
      VCR.use_cassette("rex_get_page_401", :match_requests_on => [:method, :uri, :path]) do
        client.token = "old_token"
        page = client.get_page({}, 'entity', 1)
        expect(page[:items].length).to eq 10
      end 
    end
  end 

  describe 'traverse_structures' do
    let!(:client) { Hermes::RexClient::Client.new }

    before do 
      client.base_url = "https://idrex-int.be-idys.com/id-rex-front-ws/context/1bf39a51-d57b-43fe-ad50-d426b9533b22"
      client.ed_entity = "804d1ddb-b508-40fa-8b00-27be5aba1bce"
      client.ed_lessor = "64467464-8890-4429-94fa-e0ec34134dc5"
      client.ed_contact = "864e35c1-862a-4e8e-8f13-deab5ccdbe73"
      client.client_id = "id-rex_back"
      client.client_secret = "6f593606-2306-4617-82b4-d0061a0f2d04"
      client.password = "beysALSREX2024"
      client.username = "idrex_test_user_user_rw"
      client.logger = Logger.new('log/rex-test.log')
    end 

    it 'works' do 
      VCR.use_cassette("rex_traverse_structures", :match_requests_on => [:method, :uri, :path]) do
        client.traverse_structures({'traceability.lastModifiedDate' => ">2024-03-01"},{ 
          dont_update_lessor: false, 
          dont_update_entity: false
          }, Proc.new do |rex_entity| 
            expect(rex_entity).not_to be_empty
          end
        )
      end
    end 

    it 'when receiving an error first page' do
      allow(client).to receive(:get_page).and_return({error: "an error"})

      expect{client.traverse_structures({'traceability.lastModifiedDate' => ">2024-03-01"})}.to raise_error(StandardError, "No total")
    end 

    it 'when receiving an error on page' do
      filter = {'traceability.lastModifiedDate' => ">2024-03-01"}
      allow(client).to receive(:get_page).with(filter, "lessor", 1).and_return({total: 15, items: []})
      allow(client).to receive(:get_page).with(filter, "lessor", 2).and_return({error: "an error"})
      allow(client).to receive(:log_puts)

      client.traverse_structures(filter, {dont_update_entity: true}, 
        Proc.new do |rex_entity| 
          expect(rex_entity).not_to be_empty
        end
      )

      expect(client).to have_received(:log_puts).with("an error").exactly(1).times
    end 
  end   
end
