require 'rails_helper'

RSpec.describe BatchTasks::Base, type: :service do
  let!(:log_name) { 'test_log_file.log' }

  subject { described_class.new(log_name) }

  after do
    File.delete(log_name) if File.exist?(log_name)
  end

  describe '#create_log_doc' do
    let(:tagset_id) { 'tagset_test' } 

    context 'when the CSV file exists' do
      before do
        File.open(log_name, 'w') { |file| file.write('Test content') }
      end

      it 'reads content from the CSV file and assigns it to the document blob' do
        doc = subject.create_log_doc(tagset_id)
        expect(doc.data).to include('Test content')
        expect(doc.tagset_ids).to include(tagset_id)
      end

      it 'deletes the CSV file after reading' do
        subject.create_log_doc(tagset_id)
        expect(File.exist?(log_name)).to be_falsey
      end

      it 'clearn line and write it the doc' do
        subject.update_log_lines(['Other test content'])
        doc = subject.create_log_doc(tagset_id)
        expect(subject.log_lines).to be_empty
        expect(doc.data).to include('Other test content')
      end 
    end
  end

  describe '#add_to_log_file' do
    let(:line) { 'simple data' }

    context 'when adding a line' do
      it 'appends the line to the log_lines' do
        subject.add_to_log_file(line)
        expect(subject.log_lines).to include(line)
      end
    end

    context 'when log_lines reaches 1000 lines or force_flush is true' do
      before { 1000.times { subject.add_to_log_file(line) } }

      it 'writes the content to the CSV file and clears the buffer' do
        expect(File).to exist(log_name)
        expect(subject.log_lines).to be_empty
        file_content = File.read(log_name)
        expected_content = 1000.times.map { line + "\n" }.join
        expect(file_content).to include(expected_content)
      end
    end
  end

  describe '#print_time' do
    let(:time) { Time.new(2023, 1, 1, 12, 0, 0) }

    it 'formats the time correctly' do
      expect(subject.print_time(time)).to eq('2023-01-01-11:00:00(UTC)')
    end
  end
end